@@ -77,9 +77,6 @@
 #define DEF_SEND_SECS	0
 #define DEF_DATA_LEN	40	/*  all but IP header...  */
 #define MAX_PACKET_LEN	65000
-#ifndef DEF_AF
-#define DEF_AF		AF_INET
-#endif
 
 #define ttl2hops(X)	(((X) <= 64 ? 65 : ((X) <= 128 ? 129 : 256)) - (X))
 
@@ -224,9 +221,7 @@ static int getaddr (const char *name, sockaddr_any *addr) {
 	}
 
 	for (ai = res; ai; ai = ai->ai_next) {
-	    if (ai->ai_family == af)  break;
-	    /*  when af not specified, choose DEF_AF if present   */
-	    if (!af && ai->ai_family == DEF_AF)
+	    if (!af || ai->ai_family == af)
 		    break;
 	}
 	if (!ai)  ai = res;	/*  anything...  */
